<?php
/*************************************************************************/
/* SPAGHETTILEARNING - E-Learning System                                 */
/* ============================================                          */
/*                                                                       */
/* Copyright (c) 2004 by Emanuele Sandri (esandri@tiscali.it)  			 */
/*																		 */
/* This program is free software. You can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 2 of the License.        */
/*************************************************************************/

/**
 * @module CPManager.php
 *
 * This module is the content package manager of the SpaghettiScorm
 *
 * @author Emanuele Sandri
 * @version $Id: RendererBase.php,v 1.3 2004/06/02 23:51:44 ema Exp $
 * @copyright 2004
 **/

/**
 *  RendererAbstract is the prototype class for all the
 *  Renderer objects
 */
class RendererAbstract {

	function RenderStartItem( $this, $itemInfo ){
	}
	
	function RenderStopItem( $this, $itemInfo ){
	}
}

/**
	|-----|-------------------------------------------------|
	|(0.a)|root (1.i)                                       |
	|-----|-----|-------------------------------------------|
	|(0.f)|(1.b)|child1 (2.i)                             	|
	|-----|-----|-----|-------------------------------------|
	|(0.f)|(1.c)|(2.d)|child1.1 (3.i)                       |
	|-----|-----|-----|-------------------------------------|
	|(0.f)|(1.c)|(2.e)|child1.2 (3.i)                       |
	|-----|-----|-----|-------------------------------------|
	|(0.f)|(1.a)|child2 (2.i)                               |
	|-----|-----|-----|-------------------------------------|
	|(0.f)|(1.h)|child3 (2.i)                               |
	|-----|-----|-----|-------------------------------------|
	|(0.f)|(1.f)|(2.h)|child3.1 (3.i)                       |
	|-----|-----|-----|-----|-------------------------------|
	|(0.f)|(1.f)|(2.f)|(3.e)|child3.1.1 (4.i)               |
	|-----|-----|-----|-----|-------------------------------|

	+   intermezzo	n.a
	-   intermezzo	n.b
	|   			n.c
	|-  			n.d
	L   			n.e
	    vuoto		n.f
	+   finale      n.g
	-   finale      n.h
	    titolo      n.i
	    
*/

/*define("REND_EXPAND_INTER", "a");
define("REND_COLLAPSE_INTER", "b");
define("REND_VERT_INTER", "c");
define("REND_BRANCH_INTER", "d");
define("REND_BRANCH_END", "e");
define("REND_EMPTY", "f");
define("REND_EXPAND_END", "g");
define("REND_COLLAPSE_END", "h");
define("REND_TITLE", "i");*/

define("REND_EXPAND_INTER", "menu_tee_plus.gif");
define("REND_COLLAPSE_INTER", "menu_tee_minus.gif");
define("REND_VERT_INTER", "menu_bar.gif");
define("REND_BRANCH_INTER", "menu_tee.gif");
define("REND_BRANCH_END", "menu_corner.gif");
define("REND_EMPTY", "menu_pixel.gif");
define("REND_EXPAND_END", "menu_corner_plus.gif");
define("REND_COLLAPSE_END", "menu_corner_minus.gif");
define("REND_TITLE", "menu_folder_open.gif");

class RendererDefaultImplementation extends RendererAbstract {

	// 6 class for any deep
	var $xClasses = array();
 	var $stack;
 	var $deep = 0;
	var $row = 0;
 	var $classPrefix = "ElemTree_";
	var $imgPrefix = "";
	var $imgOptions = "";
	var $resBase = "";
	var $showlinks = true;
	var $showit = false;
	var $itemtrack = null;
	var $idUser = FALSE;
	var $renderStatusCallBack = FALSE;
	
	function RendererDefaultImplementation() {
        $this->xClasses[] = array( '0.a', '0.b', '0.c', '0.d', '0.e', '0.f', '0.g', '0.h', '0.i' );
	}

	function getRowClass() {
		return 'TreeRowClass';
	}

	/**
	 *  @param $deep actual deep
	 *  @param $deepPos position to get
	 *  @retun name of the stylesheet class
	 */
	function getClass( $deep, $deepPos ) {
	    $classIndex = $deepPos;
	    $classLabel = '';
		/* find correct position in xClasses array */
		if( $classIndex >= count( $this->xClasses ) )
		    $classIndex = count( $this->xClasses ) - 1;

		if( $deep == $deepPos ) {
			// handle REND_TITLE
			$classLabel = REND_TITLE;
		} else if( $deep == $deepPos + 1 ) {
			// handle REND_EXPAND_INTER,REND_COLLAPSE_INTER,
			// REND_EXPAND_END,REND_COLLAPSE_END
			// REND_BRANCH_INTER,REND_BRANCH_END    // inLeaf
			if( $this->stack[$deep]['isLeaf'] ) {
				if( $this->stack[$deep]['isLast'] ) {
                    $classLabel = REND_BRANCH_END;
				} else {
                    $classLabel = REND_BRANCH_INTER;
				}
			} else {
				if( $this->stack[$deep]['isLast'] ) {
                    $classLabel = REND_COLLAPSE_END;
				} else {
                    $classLabel = REND_COLLAPSE_INTER;
				}
			}
		} else {
			// handle REND_VERT_INTER,REND_EMPTY
			if( $this->stack[$deepPos]['isEnd'] )
			    $classLabel = REND_EMPTY;
			else
			    $classLabel = REND_VERT_INTER;
		}
		
		return $this->classPrefix . $classIndex . $classLabel;
	}

		/**
	 *  @param $deep actual deep
	 *  @param $deepPos position to get
	 *  @retun name of the stylesheet class
	 */
	function getImage( $deep, $deepPos ) {
	    $imgLabel = '';
	
		if( $deep == $deepPos ) {
			// handle REND_TITLE
			$imgLabel = REND_TITLE;
		} else if( $deep == $deepPos + 1 ) {
			// handle REND_EXPAND_INTER,REND_COLLAPSE_INTER,
			// REND_EXPAND_END,REND_COLLAPSE_END
			// REND_BRANCH_INTER,REND_BRANCH_END    // inLeaf
			if( $this->stack[$deep]['isLeaf'] ) {
				if( $this->stack[$deep]['isLast'] ) {
                    $imgLabel = REND_BRANCH_END;
				} else {
                    $imgLabel = REND_BRANCH_INTER;
				}
			} else {
				if( $this->stack[$deep]['isLast'] ) {
                    $imgLabel = REND_COLLAPSE_END;
				} else {
                    $imgLabel = REND_COLLAPSE_INTER;
				}
			}
		} else {
			// handle REND_VERT_INTER,REND_EMPTY
			if( $this->stack[$deepPos]['isEnd'] )
			    $imgLabel = REND_EMPTY;
			else
			    $imgLabel = REND_VERT_INTER;
		}
		
		return $this->imgPrefix . $imgLabel;
	}

	/**
	 *  @param $cpm reference to CPManager
	 *  @param $itemInfo info array:
	 *      'identifier'
	 *      'isLast'
	 *      'identifierref'
	 *      'isvisible'
	 *      'parameters'
	 *      'title'
	 *      'isLeaf'
	 */
	function RenderStartItem( $cpm, $itemInfo ){
		// Add some info to $itemInfo hash array
		$itemInfo['isEnd'] = FALSE;                 // the branch is not ended
		$itemInfo['idRow'] = $this->row;            // identifier of row
  		$this->stack[$this->deep]['nChild'] = 0;    // number of renderd child
		if($this->deep > 0) {
            // increase the parent's number of childs
        	$this->stack[$this->deep-1]['nChild']++;
        	// set the sequence id. Progressive number in the set of siblings
        	$itemInfo['idSeq'] = $this->stack[$this->deep-1]['nChild'];
		} else {
            // set the sequence id. The root is always 1
            $itemInfo['idSeq'] = 1;
		}
		
		// store $itemInfo in a stack (array) for next usage
		$this->stack[$this->deep] = $itemInfo;

		if( $this->showit ) {
			if ($this->deep == 0)
			    $rs = $this->itemtrack->getItemTrack($this->idUser, null, $itemInfo['uniqueid'] );
			else 
				$rs = $this->itemtrack->getItemTrack($this->idUser, $itemInfo['uniqueid'], null );
			
			if ( $rs === FALSE ) {
			    echo "<div class=\"reportincomplete\">Never started</div>";
			} else {
				$report = mysql_fetch_assoc($rs);
				if( $this->renderStatusCallBack === FALSE ) {
					switch($report['status']) {
						case 'completed': {
							$class = "reportcomplete";
							$text = "completed";
						} break;
						case 'passed': {
							$class = "reportcomplete";
							$text = "passed: ".$this->itemtrack->getTrackingScore($report['idscorm_tracking'])."/".$itemInfo['adlcp_masteryscore'];
						} break;
						case 'failed': {
							$class = "reportincomplete";
							$text = "failed: ".$this->itemtrack->getTrackingScore($report['idscorm_tracking'])."/".$itemInfo['adlcp_masteryscore'];
						} break;
						default: {
							$class = "reportincomplete";
							$text = $report['status'];
						} break;
					}
					echo "<div class=\"$class\">$text</div>";
				} else {
					$cback = $this->renderStatusCallBack;
					if( $report['status'] == 'passed' || $report['status'] == 'failed' )
						$cback($report['status'],$this->itemtrack->getTrackingScore($report['idscorm_tracking']),$itemInfo['adlcp_masteryscore']);
					else 
						$cback($report['status']);
				}
			}
		}
		
		// start a new level
		// one block for any deep
  		echo '<span class="'. $this->getRowClass() .'"'
		  	.' id="';
  		for( $ideep = 0; $ideep < $this->deep; $ideep++ )
  		    echo $this->stack[$ideep]['idSeq'] . '.';
		echo $this->stack[$this->deep]['idSeq'];
		echo '">'. "\n";
		for( $deepIndex = 0; $deepIndex < $this->deep; $deepIndex++ ) {
			echo '	<img class="TreeClass" src="'. $this->getImage( $this->deep, $deepIndex )
			.'" '. $this->imgOptions .' >'. "\n";
		}
			
		if( !$itemInfo['isLeaf'] ) {
			echo '	<img class="TreeClass" src="'. $this->getImage( $this->deep, $deepIndex )
			.'" '. $this->imgOptions .' >'. "\n";
		}
		
		if( $itemInfo['identifierref'] ) {
			$resInfo = $cpm->GetResourceInfo($itemInfo['identifierref']);
			if( $itemInfo['prerequisites']) {
				if( $this->showlinks ) {
					echo '<a href="#" id="link'.$itemInfo['idRow']
						.'" class="accessible"'
						.' onclick="LoadSco(this,\''
						.$resInfo['uniqueid'].'\',\''
						.$itemInfo['uniqueid'].'\',\''
						.$this->stack[0]['uniqueid'].'\',\''
						.$resInfo['scormtype'] 
						.'\'); return false;">'
						.$itemInfo['title']
						.'</a>';
				} else {
					echo $itemInfo['title'];				
				}
			} else {
				if( $this->showlinks ) {
					echo '<a href="#" id="noaccess'.$itemInfo['idRow'].'" class="notaccessible" onclick="alert(\'The prerequisites are not satisfied!\'); return false;">'
					.$itemInfo['title']
					.'</a>';
				} else {
					echo $itemInfo['title'];				
				}
			}
		} else {
			echo '<span id="treenode'.$itemInfo['idRow'].'">'.$itemInfo['title'].'</span>'."\n";
		}
		echo '</span>'. "<br />\n";

		
		//echo '<br />'. "\n";
		// if this is the last child then the parent is ended
		if( $itemInfo['isLast'] && $this->deep > 0 )
		    $this->stack[$this->deep - 1]['isEnd'] = TRUE;

		$this->deep++;
		$this->row++;
	}

	function RenderStopItem( $cpm, $itemInfo ){
		//echo '<!-- RenderStopItem ';
		//print_r($itemInfo);
		//echo '-->';
		$this->deep--;
	}
	
}


?>
