<?php
/*************************************************************************/
/* SPAGHETTILEARNING - E-Learning System                                 */
/* ============================================                          */
/*                                                                       */
/* Copyright (c) 2004 by Emanuele Sandri (esandri@tiscali.it)  			 */
/*																		 */
/* This program is free software. You can redistribute it and/or modify  */
/* it under the terms of the GNU General Public License as published by  */
/* the Free Software Foundation; either version 2 of the License.        */
/*************************************************************************/

/**
 * @module sorm_utils.php
 * Utilities functions
 *
 * @version $Id: scorm_utils.php,v 1.5 2004/07/28 16:27:12 ema Exp $
 * @copyright 2004 
 **/

include_once( dirname(__FILE__) . '/../../config.php' );
require_once( dirname(__FILE__) . '/config.scorm.php');
 
/*
 * Cerca tra gli elementi del nodo il primo elemento che ha tag $name
 * ed un attributo con nome $attribname e valore $attribvalue 
 * se $attribname == "" && $attribvalue == "" non c'e' ricerca tra gli 
 * attributi.
 * Torna l'elemento trovato o NULL.
 */
function getelementbynameattrib(&$node, $name, $attribname = "", $attribvalue = ""){
	$elem = $node->first_child();
	while( $elem ) {
		if( ($elem->node_type() == XML_ELEMENT_NODE) && ($elem->node_name() == $name) ) {
			if( $attribname != "" ) {
				if( $elem->get_attribute($attribname) == $attribvalue) 
					break;	// trovato
			} else 
				break; // trovato
		}
		$elem = $elem->next_sibling();	// successivo elemento
	}
	return $elem;
}

function translateParam($parampath)
{
	$path = explode('.', $parampath);
	return $path;
} 

function getXmlDocTrackingTemplate() {
	//$pathToFile = $_SERVER['PATH_TRANSLATED'];
	//$pathToFile = stripslashes(substr($pathToFile, 0, strrpos($pathToFile, '\\')+1));
	//$pathToFile = 'trackingdatamodel-1.2.xml';
	//$pathToFile = 'modules/scorm/trackingdatamodel-1.2.xml';
    $pathToFile = dirname(__FILE__). '/trackingdatamodel-1.2.xml';
	$xmldoc = domxml_open_file($pathToFile)
		or die("Error opening template file: $pathToFile<br/>"
		."Directory di lavoro: ". getcwd());
	return $xmldoc;
}

function getXmlElementTrackingTempalte(&$base, $pathelem) {
	$path = translateParam($pathelem);
	$elem = $base;
	$increment = 1;
	for($i = 0; $i < count($path) && $elem; $i += $increment) {
		$step = $path[$i];
		if ($i < (count($path)-1))
			$next = $path[$i + 1];
		else
			$next = "";
		$index = "";
		$increment = 1; 
		
		// Virtual elements
		if( $step == '_count' 
		||	$step == '_children')
			return false;

		// if next step is a number the element to find is an
		// element of a list
		if (is_numeric($next)) {
			$index = $next;
			$increment = 2;
			$elem = getelementbynameattrib($elem, $step, "index", "yes");
		} else {
			$elem = getelementbynameattrib($elem, $step );
		}
	}
	return $elem;
}

function getElementsArrayTrackingTemplate($pathelem) {
	$path = translateParam($pathelem);
	$xmldoc = getXmlDocTrackingTemplate();
	$elem = $xmldoc->document_element();
	$increment = 1;
	$resultArray = array();
	for($i = 0; $i < count($path) && $elem; $i += $increment) {
		$step = $path[$i];
		if ($i < (count($path)-1))
			$next = $path[$i + 1];
		else
			$next = "";
		$index = "";
		$increment = 1; 
		
		// Virtual elements
		switch( $step ) {
			case '_count':
				if( strlen($next) > 0 ) { 
					$resultArray[] = array($step, false); // after _count can't be anything
				} elseif( $elem->get_attribute('index') != 'yes' ) {
					$resultArray[] = array($step, false); // count on an uncountable element 
				} else {
					$resultArray[] = array($step, true); // ok 
				}
				return $resultArray;
			break;
			case '_children':
				if( strlen($next) > 0 ) {
					$resultArray[] = array($step, false); // after _children can't be anything
				} elseif( $elem->get_attribute('item') == 'yes' ) {
					$resultArray[] = array($step, false); // can't get children of a terminal element
				} else {
					$resultArray[] = array($step, true); // ok
				}
				return $resultArray;
			break;
			case '_version':
				$step = 'version'; // the element name in template is 'version'
			break;
		}

		// if next step is a number the element to find is an
		// element of a list
		if (is_numeric($next)) {
			$index = $next;
			$increment = 2;
			$name = $index;
			$elem = getelementbynameattrib($elem, $step, "index", "yes");
		} elseif (strlen($next)>0) {
			// it's not the last element
			$elem = getelementbynameattrib($elem, $step, "item", "no" );
			$name = $step;
		} else {
			// last element
			$elem = getelementbynameattrib($elem, $step, "item", "yes" );
			$name = $step;
		}
		if( $elem )
			$resultArray[]=array($name,$elem);
		else
			$resultArray[]=array($name,false);
	}
	return $resultArray;
}

/**
 * Create the run-time environment
 * @param $userid id of the user
 * @param $scoid id of the sco
 * @param $cpid id of the content package
 * @param $scormws web service for scorm 
 * 			(e.g. "http://127.0.0.1/splearning/modules/scorm/soaplms.php" )
 * @param $scromserviceid identifier of the service (e.g. "urn:SOAPLMS")
 */
function createSCORM_RunTimeEnvironment( $userid, $scoid, $idscormpackage, $scormws, $scormserviceid ){
	$debugScormAPI = false;
	if( $debugScormAPI ) {
		echo '<div id="dbgWindow" style="overflow:scroll;width:390;height:400">';
		echo '</div>';
	}
	
	echo '<SCRIPT language="JavaScript" src="modules/scorm/ScormApi.js"></SCRIPT>'."\n";
	echo '<SCRIPT language="JavaScript">'."\n";
	echo '<!--'."\n";
	echo 'window.API = new ScormApiUI( "'. $_SERVER['HTTP_HOST'] .'",'
		.'"'. $scormws. '",'
		.'"'. $scormserviceid. '",' 
		.'"'. $userid. '",' 
		.'"'. $scoid. '",'
		.'"'. $idscormpackage. '");'."\n";
		
	echo 'window.parent.API = window.API;'."\n";
	echo 'window.top.API = window.API;'."\n";
	if( $debugScormAPI ) {
		echo 'window.API.dbgLevel = 1;';
		echo 'window.API.dbgOut = document.getElementById("dbgWindow");';
	}
	echo 'function LoadSco( scoid, itemid, orgid ) { '."\n";
	echo '	window.API.setScoid( scoid );'."\n";
	echo '	window.API.setItemid( itemid );'."\n";
	echo '	window.API.setOrgid( orgid );'."\n";
	echo '	window.open( "index.php?op=scoload&userid='.$userid.'&scoid=" + scoid + "&itemid=" + itemid + "&orgid=" + orgid + "&idscormpackage='.$idscormpackage.'" );'."\n";
	echo '}'."\n";
	echo ' // -->'."\n";
	echo '</SCRIPT>'."\n";
}

function sumScormTime( $t1, $t2 ) {
	// HHHH:MM::SS.SS
	$a1 = explode(":", $t1);
	$h1 = $a1[0];
	$m1 = $a1[1];
	$pos = strpos($a1[2], ".");
	if( $pos === FALSE ) {
		$s1 = $a1[2];
		$c1 = 0;
	} else {
		$s1 = substr($a1[2], 0, $pos);
		$c1 = substr($a1[2], $pos+1);
	}
	
	$a2 = explode(":", $t2);
	$h2 = $a2[0];
	$m2 = $a2[1];
	$pos = strpos($a2[2], ".");
	if( $pos === FALSE ) {
		$s2 = $a2[2];
		$c2 = 0;
	} else {
		$s2 = substr($a2[2], 0, $pos);
		$c2 = substr($a2[2], $pos+1);
	}
	
	$h = 0;
	$m = 0;
	$s = 0;
	$c = 0;
	
	$c += $c1 + $c2;
	if( $c >= 100 ) {
		$s++;
		$c -= 100;
	}
	$s += $s1 + $s2;
	if( $s >= 60 ) {
		$m++;
		$s -= 60;
	}
	$m += $m1 + $m2;
	if( $m >= 60 ) {
		$h++;
		$m -= 60;
	}
	$h += $h1 + $h2;
	
	return sprintf("%04u:%02u:%02u.%02u", $h, $m, $s, $c);
}

function delDirTree( $path ) {
	
	$dirobj = dir($path);
	while( false !== ($entry = $dirobj->read())) {
		if (($entry!=".")&&($entry!="..")) {
			$currpath = $dirobj->path.'/'.$entry;
			if( is_dir($currpath) ) {
				delDirTree($currpath);
			} else {
				unlink($currpath);
			}
		}
	}
	$dirobj->close();
	rmdir($path);
}

/**
 * isTrackingAvailable()
 * 
 * @param $idscorm_package
 * @return boolean TRUE if there are some tracking for this package,
 * 			FALSE otherwise
 **/
function isTrackingAvailable( $idscorm_package, $idProg ) { 
	global $prefix;
	if( $idscorm_package == NULL ) {
		$query = "SELECT st.idscorm_tracking"
				." FROM ".$prefix."_scorm_tracking st,".$prefix."_scorm_resources sr,".$prefix."_scorm_package sp"
				." WHERE sp.idProg=".$idProg
				." AND sp.idscorm_package=sr.idscorm_package"
				." AND sr.idscorm_resource=st.idscorm_resource";
	} else {
		$query = "SELECT st.idscorm_tracking"
				." FROM ".$prefix."_scorm_tracking st,".$prefix."_scorm_resources sr"
				." WHERE sr.idscorm_package=".$idscorm_package 
				." AND sr.idscorm_resource=st.idscorm_resource";
	}
	// debug echo "<!-- isTrackingAvailable:Query = $query -->";
	$rs = mysql_query($query) or die(mysql_error());
	return (mysql_num_rows($rs)>0)?TRUE:FALSE;
}

function getPackIdOrgIdFromProgId( $progId, $prefix ) {
	$query = "SELECT so.idscorm_package, so.idscorm_organization "
			." FROM ".$prefix."_scorm_package sp"
			.",".$prefix."_scorm_organizations so"
			." WHERE sp.idProg=".$progId
			." AND sp.idscorm_package=so.idscorm_package"
			." AND sp.defaultOrg=so.org_identifier";
	$rs = mysql_query($query) or die(mysql_error());
	return mysql_fetch_row($rs);
}

$scorm_pre_patterns[] = '/~/';
$scorm_pre_patterns[] = '/&/';
$scorm_pre_patterns[] = '/\|/';
$scorm_pre_patterns[] = '/(?<=[\x2C&|!({^])\s*([a-z,A-Z,0-9,_,\-]+)\s*(?=[\x2C}&|!);])/';
$scorm_pre_patterns[] = '/([a-z,A-Z,0-9,_,\-]+)\s*=\s*"([a-z,A-Z,0-9,_,\-]+)"/';
$scorm_pre_patterns[] = '/([a-z,A-Z,0-9,_,\-]+)\s*<>\s*"([a-z,A-Z,0-9,_,\-]+)"/';
$scorm_pre_patterns[] = '/(\d+)\s*\*\s*\{(.*)\}/';

$scorm_pre_replacements[] = '!';
$scorm_pre_replacements[] = '&&';
$scorm_pre_replacements[] = '||';
$scorm_pre_replacements[] = '$this->isOk_scoid("\\1")';
$scorm_pre_replacements[] = '$this->getValue_scoid("\\1")=="\\2"';
$scorm_pre_replacements[] = '$this->getValue_scoid("\\1")!="\\2"';
$scorm_pre_replacements[] = '$this->testArray_scoid(\\1, array(\\2))';

class SCORM12_Sequencing {

	var $idUser;
	var $idscorm_package;
	var $dbconn;
	var $prefix;
	
	function SCORM12_Sequencing( $idscorm_package, $idUser, $dbconn, $prefix ) {
		if( $idUser === FALSE ) 
			$this->idUser = sl_sal_getUserId();
		else 
			$this->idUser = $idUser;
		
		$this->idscorm_package= $idscorm_package;
		$this->dbconn = $dbconn;
		$this->prefix = $prefix;
	}

	function isOk_scoid( $scoid ) {
		$status = $this->merge_status($this->getValue_scoid( $scoid ));
		if ( (strcmp($status,"completed")==0) || (strcmp($status,"passed") == 0) ) {
		    return TRUE;
		} else {
			return FALSE;
		}
	}
	
	function merge_status( $arrStatus ) {
		// debug echo "\n<!-- arrStatus ";
		// debug print_r($arrStatus);
		// debug echo "\n -->";
		if( !is_array($arrStatus) ) 
			return $arrStatus;

		foreach( $arrStatus as $currStatus ) {
			if( (strcmp($currStatus,"completed")!=0) && (strcmp($currStatus,"passed") != 0) )
				return "incomplete";
		}
		return "completed";
	}
	
	function getValue_scoid( $scoid, $idscrom_item = NULL ) {
		// test if scoid is a block item 
		// this test can be done in two ways:
		// 1) the item identifierref is null
		// 2) the item as some children
		// Now I implement the second 
		if( $idscrom_item === NULL ) {
			$query = "SELECT si.idscorm_item, si.identifierref"
					." FROM ".$this->prefix."_scorm_items si,".$this->prefix."_scorm_organizations so"
					." WHERE so.idscorm_package=".$this->idscorm_package
					." AND so.idscorm_organization=si.idscorm_organization"
					." AND si.item_identifier='$scoid'";
		} else {
			$query = "SELECT si.idscorm_item, si.identifierref"
					." FROM ".$this->prefix."_scorm_items si"
					." WHERE si.idscorm_item=".$idscrom_item;
		}
		$rsItem = mysql_query($query, $this->dbconn);
		list($idscorm_item, $identifierref) = mysql_fetch_row($rsItem);
		if(strlen($identifierref) == 0) {
			// now i must find all the child items and call this function recursively
			$query = "SELECT idscorm_item"
					." FROM ".$this->prefix."_scorm_items"
					." WHERE idscorm_parentitem=".$idscorm_item;
			$rs = mysql_query($query, $this->dbconn);
			$resultarray = array();
			while( FALSE !== (list($idsubitem) = mysql_fetch_row($rs))) 
				$resultarray = array_merge($resultarray, $this->getValue_scoid(NULL, $idsubitem));
			return $resultarray;
		} else {
			$query = "SELECT st.lesson_status"
					." FROM ".$this->prefix."_scorm_tracking st,".$this->prefix."_scorm_resources sr"
					." WHERE sr.idscorm_package=".$this->idscorm_package
					." AND sr.idscorm_resource=st.idscorm_resource"
					." AND sr.idsco='$identifierref'"
					." AND st.idUser=".$this->idUser;
			$rs = mysql_query($query, $this->dbconn);
			if( $rs === FALSE ) {
				echo "\n<!-- SCORM12_Sequencing::getValue_scoid error on query $query\n"
					.mysql_error($this->dbconn). " -->\n";
				return FALSE;
			}
			if( mysql_num_rows($rs) == 0) {
				//echo "<!-- SCORM12_Sequencing::getValue_scoid return not attempted -->";
				return "not attempted";
			} else {
				list( $status ) = mysql_fetch_row($rs);
				//echo "<!-- SCORM12_Sequencing::getValue_scoid return $status -->";
				return $status;
			}					
		}


		
		/*
		* 		$query = "SELECT st.lesson_status"
				." FROM ".$this->prefix."_scorm_tracking st,".$this->prefix."_scorm_resources sr,"
				.$this->prefix."_scorm_items si,".$this->prefix."_scorm_organizations so"
				." WHERE so.idscorm_package=".$this->idscorm_package
				." AND so.idscorm_organization=si.idscorm_organization"
				." AND si.identifierref=sr.idsco"
				." AND sr.idscorm_package=".$this->idscorm_package
				." AND sr.idscorm_resource=st.idscorm_resource"
				." AND si.item_identifier='$scoid'"
				." AND st.idUser=".$this->idUser;
				* */
	}
	
	function testArray_scoid( $nComplete, $array_values ) {
		$nTotComplete = 0;
		foreach( $array_values as $val ) {
			if( $val )
			    $nTotComplete++;
		}
		return $nTotComplete >= $nComplete;
	}
			
	function evauatePrerequisites( $prerequisites ) {
		global $scorm_pre_replacements,$scorm_pre_patterns;
		// debug echo "<!-- evauatePrerequisites $prerequisites\n";
		// debug print_r($scorm_pre_replacements);
		// debug echo " -->";
		$cmdString = 'return ' . preg_replace( $scorm_pre_patterns , $scorm_pre_replacements, '('.$prerequisites.');');
		echo "<!-- command string is $cmdString -->";
		return eval($cmdString);
	}

}


?>
